<?php
// src/Controller/LuckyController.php
namespace App\Controller;

use App\Entity\User;
use App\ORM\ORM;

use Doctrine\ORM\EntityManagerInterface;

use Symfony\Component\Mailer\MailerInterface;
use Symfony\Component\Mime\Email;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpFoundation\BinaryFileResponse;
use Symfony\Component\HttpFoundation\ResponseHeaderBag;
use Symfony\Component\DependencyInjection\ParameterBag\ParameterBagInterface;
use Symfony\Component\Security\Core\Security;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\Routing\Annotation\Route;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\Request;

use TheNetworg\OAuth2\Client\Provider\Azure;

use Doctrine\ORM\Query\Expr\Func;
use Microsoft\Kiota\Authentication\Oauth\ClientCredentialContext;
use Microsoft\Graph\Core\Authentication\GraphPhpLeagueAuthenticationProvider;
use Microsoft\Graph\GraphServiceClient;
use Microsoft\Graph\GraphServiceRequest;
use League\OAuth2\Client\Provider\Exception\IdentityProviderException;
use League\OAuth2\Client\Provider\GenericProvider;
use Twilio\Rest\Client;

class inicioController extends AbstractController
{
    private $orm;
    private $rutaFichero;

    private $tokenData;
    private $aInicio;
    private $sInquilino;

    const rutaAPP = 'https://pre.atlanticomedio.es/aplicaciones';
    const rutaHome = 'https://pre.atlanticomedio.es/';


    public function __construct(EntityManagerInterface $entityManager, ParameterBagInterface $params)
    {
        $this->orm = new ORM($entityManager);
        $this->rutaFichero = $params->get('rutaDirectorio');
        $this->sInquilino =  $params->get('INQUILINO');


        $clienteID = $params->get('CLIENTE_ID');
        $clienteSecret = $params->get('CLIENTE_SECRET');
        $rutaMS = $params->get('RUTA');

        $this->tokenData = [
            'grant_type' => 'client_credentials',
            'client_id' => $clienteID,
            'client_secret' => $clienteSecret,
            'resource' => 'https://graph.microsoft.com',
        ];

        $this->aInicio = [
            'clientId'          => $clienteID,
            'clientSecret'      => $clienteSecret,
            'redirectUri'       => $rutaMS, // tiene que ser la misma que en AZURE
            'graphApiVersion'   => 'beta', // Puedes ajustar la versión de la API de Graph según tus necesidades
        ];
    }

    #[Route('/', name: 'ms_login')]
    public function redirectToMicrosoftProvider(Request $request)
    {
        $azure = new Azure($this->aInicio);

        // Genera una URL de autorización y redirige al usuario
        $authorizationUrl = $azure->getAuthorizationUrl();

        //guardamos el estado en la SESSION
        $request->getSession()->set('oauth2state', $azure->getState());

        return new RedirectResponse($authorizationUrl);
    }

    #[Route('/MSCallBack', name: 'ms_callBack')]
    public function recibeDatosMS(Request $request)
    {

        $azure = new Azure($this->aInicio);

        // Verifica que el estado sea válido antes de intentar intercambiar tokens
        $state = $request->query->get('state');
        $estado = $request->getSession()->get('oauth2state');

        if (empty($state) || ($state !== $estado)) {
            $request->getSession()->set('oauth2state', null);
            throw new \RuntimeException('Estado de OAuth no válido');
        }

        // Intercambia el código de autorización por un token de acceso
        $token = $azure->getAccessToken('authorization_code', [
            'code' => $request->query->get('code'),
        ]);

        // Utiliza el token para obtener información del usuarioñ
        $user = $azure->getResourceOwner($token)->toArray();

        $request->getSession()->set('usuarioMS', $user);

        //Para obtener los grupos y apps de un usuario
        $datosUsuario =  $this->obtenerGrupos($user['unique_name']);

        $usuario = [];
        $usuario['nombre'] = $user['name'];
        $usuario['mail'] = $user['unique_name'];
        $usuario['ip'] = $user['ipaddr'];
        $usuario['gruposApp'] = $datosUsuario;
        /*
        $oUsuario = new User();
        $oUsuario->setId($user['name']);
        $oUsuario->setEmail($user['unique_name']);
        */

        $aAplicaciones = $this->orm->comprobarUsuarioMSLoginBD($user['unique_name'], $sError);

        //orm::depurar($aAplicaciones, false, true);

        if ($sError) {
            return $this->redirectToRoute('sin_permiso', ['mensaje' => $sError]);
        } else {
            //si todo ok, metemos el usuario en la session para usarlo en la app
            $request->getSession()->set('usuarioAplicativo', $user['name']);
            $request->getSession()->set('aUsuarioAplicativoMS', $usuario);
            $request->getSession()->set('aplicaciones', $aAplicaciones);

            return $this->redirectToRoute('_inicio', []);
        }

        //return new Response();
    }

    /**
     * obtiene los grupos y las aplicaciones de un usuario
     */
    public function obtenerGrupos($userPrincipalName)
    {

        $tenantId =  $this->sInquilino; //$this::INQUILINO; // Reemplaza con el ID de tu inquilino (tenant)   
        $tokenData = $this->tokenData;

        // Obtén el token de acceso
        $tokenEndpoint = "https://login.microsoftonline.com/$tenantId/oauth2/token";
        $accessToken = $this->obtenerCurl($tokenEndpoint, null, $tokenData);

        // Obtén los grupos del usuario
        $graphApiEndpoint = "https://graph.microsoft.com/v1.0/users/$userPrincipalName/memberOf";
        $groups = $this->obtenerCurl($graphApiEndpoint, $accessToken);

        $grupos = [];
        foreach ($groups as $group) {
            $grupos[$group->id] = $group->displayName;
        }

        //obtiene los roles del usuario
        $graphApiEndpoint = "https://graph.microsoft.com/v1.0/users/$userPrincipalName/appRoleAssignments";
        $Roles = $this->obtenerCurl($graphApiEndpoint, $accessToken);

        $aRoles = [];
        foreach ($Roles as $group) {
            $aRoles[$group->appRoleId] = $group->resourceDisplayName;
        }

        $aSalida = [];
        $aSalida['rolesApp'] = $aRoles;
        $aSalida['grupos'] = $grupos;

        return $aSalida;
    }

    /**
     * Obtiene datos de MS mediante llamadas a CURL
     */
    public function obtenerCurl($graphApiEndpoint, $accessToken = null, $tokenData = [], $bLogout = false)
    {

        $ch = curl_init($graphApiEndpoint);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);

        if ($tokenData) {
            curl_setopt($ch, CURLOPT_POST, true);
            curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query($tokenData));
        } else {
            curl_setopt($ch, CURLOPT_HTTPHEADER, [
                'Authorization: Bearer ' . $accessToken,
                'Accept: application/json',
            ]);
        }

        $response = curl_exec($ch);
        curl_close($ch);

        if ($bLogout) {
            return true;
        }

        if ($tokenData) {
            $datos = json_decode($response)->access_token;
        } else {
            $datos = json_decode($response)->value;
        }

        return $datos;
    }

    /*************************************************************************/

    #[Route('/inicio', name: '_inicio')]
    public function inicioAction(Request $request)
    {
        $usuario = $request->getSession()->get('usuarioAplicativo', null);
        $aAplicaciones = $request->getSession()->get('aplicaciones', null);
       
        //datos del usuario conectado
        $sNombre = $request->getSession()->get('usuarioAplicativo', NULL);
        return $this->render('inicio.html.twig', ['nombre' => $sNombre, 'apps'=> $aAplicaciones]);
    }

    #[Route('/entrada', name: '_entrada')]
    public function entradaAction(Request $request)
    {
        $mensaje = $request->get('mensaje', null);
        return $this->render('login.html.twig', ['mensaje' => $mensaje]);
    }

    #[Route('/salida', name: '_salida')]
    public function salidaAction(Request $request)
    {
        $request->getSession()->set('usuarioAplicativo', null);
        $request->getSession()->set('usuarioApp', null);
        $request->getSession()->set('idUsuarioAplicativo', null);
        $mensaje = $request->get('mensaje', null);

        $aUsuario = $request->getSession()->get('aUsuarioAplicativoMS', null);

        $request->getSession()->set('aUsuarioAplicativoMS', null);

        return $this->render('login.html.twig', ['usuarioMS' => $aUsuario, 'mensaje' => $mensaje]);
    }


    #[Route('/testMail', name: 'test_mail')]
    public function testEmail(MailerInterface $mailer)
    {
        $email = (new Email())
            ->from('noreply@atlanticomedio.es')
            ->to('daniel.dominguez@atlanticomedio.es')
            //->cc('cc@example.com')
            //->bcc('bcc@example.com')
            //->replyTo('fabien@example.com')
            //->priority(Email::PRIORITY_HIGH)
            ->subject('MAIL')
            ->text('ENVIADO')
            ->html('<p>LISTO</p>');

        $mailer->send($email);

        return new Response('Email enviado');
    }


    /**
     * Enviar un mail
     */
    public static function envioMail($mailer, $sConCopia, $sMailto, $sAsunto, $cuerpo)
    { // 
        $rutaApp = self::rutaAPP;
        $rutaHTML = "<a href='$rutaApp'>$rutaApp</a>";
        if ($sConCopia) {
            $email = (new Email())
                ->from('noreply@atlanticomedio.es')
                ->to($sMailto)
                ->cc($sConCopia)
                //->bcc('bcc@example.com')
                //->replyTo('fabien@example.com')
                ->priority(Email::PRIORITY_HIGH)
                ->subject($sAsunto)
                ->text($cuerpo . ' URL ' . $rutaApp)
                ->html($cuerpo . ' URL ' . $rutaHTML);
        } else {
            $email = (new Email())
                ->from('noreply@atlanticomedio.es')
                ->to($sMailto)
                //->bcc('bcc@example.com')
                //->replyTo('fabien@example.com')
                ->priority(Email::PRIORITY_HIGH)
                ->subject($sAsunto)
                ->text($cuerpo . ' RUTA ' . $rutaApp)
                ->html($cuerpo . ' RUTA ' . $rutaHTML);
        }

        $mailer->send($email);

        return true;
    }

    /**
     * Notificar por tlf
     */
    public static function enviarMensajeTlf($destino, $mensaje, &$sError)
    {
        $curl = curl_init();

        $params = array(
            'token' => 'j0yx7tovw8r2luu0',
            'to' => $destino, //tlf destino
            'body' => $mensaje
        );

        curl_setopt_array($curl, array(
            CURLOPT_URL => "https://api.ultramsg.com/instance88277/messages/chat",
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => "",
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_TIMEOUT => 30,
            CURLOPT_SSL_VERIFYHOST => 0,
            CURLOPT_SSL_VERIFYPEER => 0,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST => "POST",
            CURLOPT_POSTFIELDS => http_build_query($params),
            CURLOPT_HTTPHEADER => array(
                "content-type: application/x-www-form-urlencoded"
            ),
        ));

        $response = curl_exec($curl);
        $err = curl_error($curl);

        curl_close($curl);


        if ($err) {
            $sError =  "cURL Error #:" . $err;
            return false;
        }

        return true;
    }

    #[Route('/download/{filename}', name: 'download_file')]
    public function downloadFile(string $filename): BinaryFileResponse
    {
        $fichero = "$filename";
        // Ruta completa al archivo
        $filePath = $this->rutaFichero . "/$fichero";

        // Crea una BinaryFileResponse para el archivo
        $response = new BinaryFileResponse($filePath);

        // Establece el nombre del archivo en la cabecera de la respuesta
        $response->setContentDisposition(
            ResponseHeaderBag::DISPOSITION_ATTACHMENT,
            $fichero
        );

        return $response;
    }

    #[Route('/sinPermisos', name: 'sin_permiso')]
    public function error403(Request $request): Response
    {
        $oUsuario = $request->getSession()->get('usuarioApp', null);
        //ORM::depurar($oUsuario, true);
        return $this->render('error403.html.twig');
    }

    #[Route('/testEnvioTlf', name: 'test_envio_tlf')]
    public function testTlf(Request $request): Response
    {
        //  self::enviarMensajeTlf('678928206', '¿Cuál es el último animal que subió al arca de Noé? El del-fin.', $sError);
        $sid    = "AC3aad9694ee31f5859a3d6d048813b4dc";
        $token  = "8259048b405674b0294eabaa72107c16";
        $twilio = new Client($sid, $token);
        $tlf = "+34630573204";
        $mensaje = "Esto es una prueba 123123";

        $message = $twilio->messages
            ->create(
                "whatsapp:$tlf", // to
                array(
                    "from" => "whatsapp:+14155238886",
                    "body" => $mensaje
                )
            );

        print($message->sid);

        return new Response();
    }
}
